@@ -77,22 +77,25 @@ enum pipe_error {
 #define PIPE_BLEND_MIN               3
 #define PIPE_BLEND_MAX               4
 
-#define PIPE_LOGICOP_CLEAR            0
-#define PIPE_LOGICOP_NOR              1
-#define PIPE_LOGICOP_AND_INVERTED     2
-#define PIPE_LOGICOP_COPY_INVERTED    3
-#define PIPE_LOGICOP_AND_REVERSE      4
-#define PIPE_LOGICOP_INVERT           5
-#define PIPE_LOGICOP_XOR              6
-#define PIPE_LOGICOP_NAND             7
-#define PIPE_LOGICOP_AND              8
-#define PIPE_LOGICOP_EQUIV            9
-#define PIPE_LOGICOP_NOOP             10
-#define PIPE_LOGICOP_OR_INVERTED      11
-#define PIPE_LOGICOP_COPY             12
-#define PIPE_LOGICOP_OR_REVERSE       13
-#define PIPE_LOGICOP_OR               14
-#define PIPE_LOGICOP_SET              15  
+
+enum pipe_logicop {
+   PIPE_LOGICOP_CLEAR,
+   PIPE_LOGICOP_NOR,
+   PIPE_LOGICOP_AND_INVERTED,
+   PIPE_LOGICOP_COPY_INVERTED,
+   PIPE_LOGICOP_AND_REVERSE,
+   PIPE_LOGICOP_INVERT,
+   PIPE_LOGICOP_XOR,
+   PIPE_LOGICOP_NAND,
+   PIPE_LOGICOP_AND,
+   PIPE_LOGICOP_EQUIV,
+   PIPE_LOGICOP_NOOP,
+   PIPE_LOGICOP_OR_INVERTED,
+   PIPE_LOGICOP_COPY,
+   PIPE_LOGICOP_OR_REVERSE,
+   PIPE_LOGICOP_OR,
+   PIPE_LOGICOP_SET,
+};
 
 #define PIPE_MASK_R  0x1
 #define PIPE_MASK_G  0x2
@@ -369,6 +372,7 @@ enum pipe_flush_flags {
 #define PIPE_BIND_SHADER_RESOURCE      (1 << 19) /* set_shader_resources */
 #define PIPE_BIND_COMPUTE_RESOURCE     (1 << 20) /* set_compute_resources */
 #define PIPE_BIND_COMMAND_ARGS_BUFFER  (1 << 21) /* pipe_draw_info.indirect */
+#define PIPE_BIND_QUERY_BUFFER         (1 << 22) /* get_query_result_resource */
 
 /* The first two flags above were previously part of the amorphous
  * TEXTURE_USAGE, most of which are now descriptions of the ways a
@@ -772,6 +776,14 @@ union pipe_query_result
    struct pipe_query_data_pipeline_statistics pipeline_statistics;
 };
 
+enum pipe_query_value_type
+{
+   PIPE_QUERY_TYPE_I32,
+   PIPE_QUERY_TYPE_U32,
+   PIPE_QUERY_TYPE_I64,
+   PIPE_QUERY_TYPE_U64,
+};
+
 union pipe_color_union
 {
    float f[4];
