@@ -28,11 +28,14 @@
 #include <epoxy/gl.h>
 
 #include "util/u_memory.h"
+#include "pipe/p_defines.h"
 #include "pipe/p_state.h"
 #include "pipe/p_shader_tokens.h"
 #include "vrend_renderer.h"
 #include "vrend_object.h"
 #include "tgsi/tgsi_text.h"
+#include "vrend_debug.h"
+#include "vrend_tweaks.h"
 
 /* decode side */
 #define DECODE_MAX_TOKENS 8000
@@ -108,6 +111,14 @@ static int vrend_decode_create_shader(struct vrend_decode_ctx *ctx,
             so_info.output[i].dst_offset = (tmp >> 16) & 0xffff;
             tmp = get_buf_entry(ctx, VIRGL_OBJ_SHADER_SO_OUTPUT0_SO(i));
             so_info.output[i].stream = (tmp & 0x3);
+            so_info.output[i].need_temp = so_info.output[i].num_components < 4;
+         }
+
+         for (i = 0; i < so_info.num_outputs - 1; i++) {
+            for (unsigned j = i + 1; j < so_info.num_outputs; j++) {
+               so_info.output[j].need_temp |=
+                     (so_info.output[i].register_index == so_info.output[j].register_index);
+            }
          }
       }
       shader_offset += 4 + (2 * num_so_outputs);
@@ -190,7 +201,8 @@ static int vrend_decode_clear(struct vrend_decode_ctx *ctx, int length)
    buffers = get_buf_entry(ctx, VIRGL_OBJ_CLEAR_BUFFERS);
    for (i = 0; i < 4; i++)
       color.ui[i] = get_buf_entry(ctx, VIRGL_OBJ_CLEAR_COLOR_0 + i);
-   depth = *(double *)(uint64_t *)get_buf_ptr(ctx, VIRGL_OBJ_CLEAR_DEPTH_0);
+   double *depth_ptr = (double *)(uint64_t *)get_buf_ptr(ctx, VIRGL_OBJ_CLEAR_DEPTH_0);
+   memcpy(&depth, depth_ptr, sizeof(double));
    stencil = get_buf_entry(ctx, VIRGL_OBJ_CLEAR_STENCIL);
 
    vrend_clear(ctx->grctx, buffers, &color, depth, stencil);
@@ -335,12 +347,25 @@ static int vrend_decode_set_sampler_views(struct vrend_decode_ctx *ctx, uint16_t
    return 0;
 }
 
+static void vrend_decode_transfer_common(struct vrend_decode_ctx *ctx, struct vrend_transfer_info *info)
+{
+   info->handle = get_buf_entry(ctx, VIRGL_RESOURCE_IW_RES_HANDLE);
+   info->level = get_buf_entry(ctx, VIRGL_RESOURCE_IW_LEVEL);
+   info->stride = get_buf_entry(ctx, VIRGL_RESOURCE_IW_STRIDE);
+   info->layer_stride = get_buf_entry(ctx, VIRGL_RESOURCE_IW_LAYER_STRIDE);
+   info->box->x = get_buf_entry(ctx, VIRGL_RESOURCE_IW_X);
+   info->box->y = get_buf_entry(ctx, VIRGL_RESOURCE_IW_Y);
+   info->box->z = get_buf_entry(ctx, VIRGL_RESOURCE_IW_Z);
+   info->box->width = get_buf_entry(ctx, VIRGL_RESOURCE_IW_W);
+   info->box->height = get_buf_entry(ctx, VIRGL_RESOURCE_IW_H);
+   info->box->depth = get_buf_entry(ctx, VIRGL_RESOURCE_IW_D);
+}
+
 static int vrend_decode_resource_inline_write(struct vrend_decode_ctx *ctx, uint16_t length)
 {
-   struct vrend_transfer_info info;
    struct pipe_box box;
-   uint32_t res_handle;
-   uint32_t level, usage, stride, layer_stride, data_len;
+   struct vrend_transfer_info info;
+   uint32_t data_len;
    struct iovec dataiovec;
    void *data;
 
@@ -350,27 +375,13 @@ static int vrend_decode_resource_inline_write(struct vrend_decode_ctx *ctx, uint
    if (length + ctx->ds->buf_offset > ctx->ds->buf_total)
       return EINVAL;
 
-   res_handle = get_buf_entry(ctx, VIRGL_RESOURCE_IW_RES_HANDLE);
+   memset(&info, 0, sizeof(info));
+   info.box = &box;
+   vrend_decode_transfer_common(ctx, &info);
    data_len = (length - 11) * 4;
-   level = get_buf_entry(ctx, VIRGL_RESOURCE_IW_LEVEL);
-   usage = get_buf_entry(ctx, VIRGL_RESOURCE_IW_USAGE);
-   stride = get_buf_entry(ctx, VIRGL_RESOURCE_IW_STRIDE);
-   layer_stride = get_buf_entry(ctx, VIRGL_RESOURCE_IW_LAYER_STRIDE);
-   box.x = get_buf_entry(ctx, VIRGL_RESOURCE_IW_X);
-   box.y = get_buf_entry(ctx, VIRGL_RESOURCE_IW_Y);
-   box.z = get_buf_entry(ctx, VIRGL_RESOURCE_IW_Z);
-   box.width = get_buf_entry(ctx, VIRGL_RESOURCE_IW_W);
-   box.height = get_buf_entry(ctx, VIRGL_RESOURCE_IW_H);
-   box.depth = get_buf_entry(ctx, VIRGL_RESOURCE_IW_D);
-
    data = get_buf_ptr(ctx, VIRGL_RESOURCE_IW_DATA_START);
 
-   info.handle = res_handle;
    info.ctx_id = 0;
-   info.level = level;
-   info.stride = stride;
-   info.layer_stride = layer_stride;
-   info.box = &box;
    info.offset = 0;
 
    dataiovec.iov_base = data;
@@ -378,7 +389,7 @@ static int vrend_decode_resource_inline_write(struct vrend_decode_ctx *ctx, uint
 
    info.iovec = &dataiovec;
    info.iovec_cnt = 1;
-   return vrend_transfer_inline_write(ctx->grctx, &info, usage);
+   return vrend_transfer_inline_write(ctx->grctx, &info);
 }
 
 static int vrend_decode_draw_vbo(struct vrend_decode_ctx *ctx, int length)
@@ -634,6 +645,12 @@ static int vrend_decode_create_sampler_state(struct vrend_decode_ctx *ctx, uint3
 
    for (i = 0; i < 4; i++)
       state.border_color.ui[i] = get_buf_entry(ctx, VIRGL_OBJ_SAMPLER_STATE_BORDER_COLOR(i));
+
+   if (state.min_mip_filter != PIPE_TEX_MIPFILTER_NONE &&
+       state.min_mip_filter != PIPE_TEX_MIPFILTER_LINEAR &&
+       state.min_mip_filter != PIPE_TEX_MIPFILTER_NEAREST)
+     return EINVAL;
+
    return vrend_create_sampler_state(ctx->grctx, handle, &state);
 }
 
@@ -712,6 +729,9 @@ static int vrend_decode_create_object(struct vrend_decode_ctx *ctx, int length)
    if (handle == 0)
       return EINVAL;
 
+   VREND_DEBUG(dbg_object, ctx->grctx,"  CREATE %-18s handle:0x%x len:%d\n",
+               vrend_get_object_type_name(obj_type), handle, length);
+
    switch (obj_type){
    case VIRGL_OBJECT_BLEND:
       ret = vrend_decode_create_blend(ctx, handle, length);
@@ -759,6 +779,10 @@ static int vrend_decode_bind_object(struct vrend_decode_ctx *ctx, uint16_t lengt
    uint32_t handle = get_buf_entry(ctx, VIRGL_OBJ_BIND_HANDLE);
    uint8_t obj_type = (header >> 8) & 0xff;
 
+   VREND_DEBUG(dbg_object, ctx->grctx,
+               "  BIND %-20s handle:0x%x len:%d\n",
+               vrend_get_object_type_name(obj_type), handle, length);
+
    switch (obj_type) {
    case VIRGL_OBJECT_BLEND:
       vrend_object_bind_blend(ctx->grctx, handle);
@@ -786,6 +810,11 @@ static int vrend_decode_destroy_object(struct vrend_decode_ctx *ctx, int length)
 
    uint32_t handle = get_buf_entry(ctx, VIRGL_OBJ_DESTROY_HANDLE);
 
+   VREND_DEBUG_EXT(dbg_object, ctx->grctx,
+               uint32_t obj = (get_buf_entry(ctx, 0) >> 8) & 0xFF;
+               vrend_printf("  DESTROY %-17s handle:0x%x\n",
+                       vrend_get_object_type_name(obj), handle));
+
    vrend_renderer_object_destroy(ctx->grctx, handle);
    return 0;
 }
@@ -1028,6 +1057,22 @@ static int vrend_decode_get_query_result(struct vrend_decode_ctx *ctx, int lengt
    return 0;
 }
 
+static int vrend_decode_get_query_result_qbo(struct vrend_decode_ctx *ctx, int length)
+{
+   if (length != VIRGL_QUERY_RESULT_QBO_SIZE)
+      return EINVAL;
+
+   uint32_t handle = get_buf_entry(ctx, VIRGL_QUERY_RESULT_QBO_HANDLE);
+   uint32_t qbo_handle = get_buf_entry(ctx, VIRGL_QUERY_RESULT_QBO_QBO_HANDLE);
+   uint32_t wait = get_buf_entry(ctx, VIRGL_QUERY_RESULT_QBO_WAIT);
+   uint32_t result_type = get_buf_entry(ctx, VIRGL_QUERY_RESULT_QBO_RESULT_TYPE);
+   uint32_t offset = get_buf_entry(ctx, VIRGL_QUERY_RESULT_QBO_OFFSET);
+   int32_t index = get_buf_entry(ctx, VIRGL_QUERY_RESULT_QBO_INDEX);
+
+   vrend_get_query_result_qbo(ctx->grctx, handle, qbo_handle, wait, result_type, offset, index);
+   return 0;
+}
+
 static int vrend_decode_set_render_condition(struct vrend_decode_ctx *ctx, int length)
 {
    if (length != VIRGL_RENDER_CONDITION_SIZE)
@@ -1105,7 +1150,7 @@ static int vrend_decode_set_tess_state(struct vrend_decode_ctx *ctx,
 
 static int vrend_decode_set_shader_buffers(struct vrend_decode_ctx *ctx, uint16_t length)
 {
-   int num_ssbo;
+   uint32_t num_ssbo;
    uint32_t shader_type, start_slot;
 
    if (length < 2)
@@ -1120,10 +1165,11 @@ static int vrend_decode_set_shader_buffers(struct vrend_decode_ctx *ctx, uint16_
    if (num_ssbo < 1)
       return 0;
 
-   if (start_slot + num_ssbo > PIPE_MAX_SHADER_BUFFERS)
+   if (start_slot > PIPE_MAX_SHADER_BUFFERS ||
+       start_slot > PIPE_MAX_SHADER_BUFFERS - num_ssbo)
       return EINVAL;
 
-   for (int i = 0; i < num_ssbo; i++) {
+   for (uint32_t i = 0; i < num_ssbo; i++) {
       uint32_t offset = get_buf_entry(ctx, VIRGL_SET_SHADER_BUFFER_OFFSET(i));
       uint32_t buf_len = get_buf_entry(ctx, VIRGL_SET_SHADER_BUFFER_LENGTH(i));
       uint32_t handle = get_buf_entry(ctx, VIRGL_SET_SHADER_BUFFER_RES_HANDLE(i));
@@ -1135,7 +1181,7 @@ static int vrend_decode_set_shader_buffers(struct vrend_decode_ctx *ctx, uint16_
 
 static int vrend_decode_set_atomic_buffers(struct vrend_decode_ctx *ctx, uint16_t length)
 {
-   int num_abo;
+   uint32_t num_abo;
    uint32_t start_slot;
 
    if (length < 2)
@@ -1146,10 +1192,12 @@ static int vrend_decode_set_atomic_buffers(struct vrend_decode_ctx *ctx, uint16_
    if (num_abo < 1)
       return 0;
 
-   if (start_slot + num_abo > PIPE_MAX_HW_ATOMIC_BUFFERS)
+   if (num_abo > PIPE_MAX_HW_ATOMIC_BUFFERS ||
+       start_slot > PIPE_MAX_HW_ATOMIC_BUFFERS ||
+       start_slot > PIPE_MAX_HW_ATOMIC_BUFFERS - num_abo)
       return EINVAL;
 
-   for (int i = 0; i < num_abo; i++) {
+   for (uint32_t i = 0; i < num_abo; i++) {
       uint32_t offset = get_buf_entry(ctx, i * VIRGL_SET_ATOMIC_BUFFER_ELEMENT_SIZE + 2);
       uint32_t buf_len = get_buf_entry(ctx, i * VIRGL_SET_ATOMIC_BUFFER_ELEMENT_SIZE + 3);
       uint32_t handle = get_buf_entry(ctx, i * VIRGL_SET_ATOMIC_BUFFER_ELEMENT_SIZE + 4);
@@ -1161,7 +1209,7 @@ static int vrend_decode_set_atomic_buffers(struct vrend_decode_ctx *ctx, uint16_
 
 static int vrend_decode_set_shader_images(struct vrend_decode_ctx *ctx, uint16_t length)
 {
-   int num_images;
+   uint32_t num_images;
    uint32_t shader_type, start_slot;
    if (length < 2)
       return EINVAL;
@@ -1175,10 +1223,11 @@ static int vrend_decode_set_shader_images(struct vrend_decode_ctx *ctx, uint16_t
    if (num_images < 1) {
       return 0;
    }
-   if (start_slot + num_images > PIPE_MAX_SHADER_IMAGES)
+   if (start_slot > PIPE_MAX_SHADER_IMAGES ||
+       start_slot > PIPE_MAX_SHADER_IMAGES - num_images)
       return EINVAL;
 
-   for (int i = 0; i < num_images; i++) {
+   for (uint32_t i = 0; i < num_images; i++) {
       uint32_t format = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_FORMAT(i));
       uint32_t access = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_ACCESS(i));
       uint32_t layer_offset = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_LAYER_OFFSET(i));
@@ -1249,6 +1298,89 @@ static int vrend_decode_texture_barrier(struct vrend_decode_ctx *ctx, uint16_t l
    return 0;
 }
 
+static int vrend_decode_set_debug_mask(struct vrend_decode_ctx *ctx, int length)
+{
+   char *flagstring;
+   int slen = sizeof(uint32_t) * length;
+   uint32_t *buf;
+
+   if (length < VIRGL_SET_DEBUG_FLAGS_MIN_SIZE)
+      return EINVAL;
+
+   buf = get_buf_ptr(ctx, VIRGL_SET_DEBUG_FLAGSTRING_OFFSET);
+   flagstring = malloc(slen+1);
+
+   if (!flagstring) {
+      return ENOMEM;
+   }
+
+   memcpy(flagstring, buf, slen);
+   flagstring[slen] = 0;
+   vrend_context_set_debug_flags(ctx->grctx, flagstring);
+
+   free(flagstring);
+
+   return 0;
+}
+
+static int vrend_decode_set_tweaks(struct vrend_decode_ctx *ctx, int length)
+{
+   VREND_DEBUG(dbg_tweak, NULL, "Received TWEAK set command\n");
+
+   if (length < VIRGL_SET_TWEAKS_SIZE)
+      return EINVAL;
+
+   uint32_t tweak_id = get_buf_entry(ctx, VIRGL_SET_TWEAKS_ID);
+   uint32_t tweak_value = get_buf_entry(ctx, VIRGL_SET_TWEAKS_VALUE);
+
+   vrend_set_active_tweaks(vrend_get_context_tweaks(ctx->grctx), tweak_id, tweak_value);
+   return 0;
+}
+
+
+static int vrend_decode_transfer3d(struct vrend_decode_ctx *ctx, int length, uint32_t ctx_id)
+{
+   struct pipe_box box;
+   struct vrend_transfer_info info;
+
+   if (length < VIRGL_TRANSFER3D_SIZE)
+      return EINVAL;
+
+   memset(&info, 0, sizeof(info));
+   info.box = &box;
+   info.ctx_id = ctx_id;
+   vrend_decode_transfer_common(ctx, &info);
+   info.offset = get_buf_entry(ctx, VIRGL_TRANSFER3D_DATA_OFFSET);
+   int transfer_mode = get_buf_entry(ctx, VIRGL_TRANSFER3D_DIRECTION);
+   info.context0 = false;
+
+   if (transfer_mode != VIRGL_TRANSFER_TO_HOST &&
+       transfer_mode != VIRGL_TRANSFER_FROM_HOST)
+      return EINVAL;
+
+   return vrend_renderer_transfer_iov(&info, transfer_mode);
+}
+
+static int vrend_decode_copy_transfer3d(struct vrend_decode_ctx *ctx, int length)
+{
+   struct pipe_box box;
+   struct vrend_transfer_info info;
+   uint32_t src_handle;
+
+   if (length != VIRGL_COPY_TRANSFER3D_SIZE)
+      return EINVAL;
+
+   memset(&info, 0, sizeof(info));
+   info.box = &box;
+   vrend_decode_transfer_common(ctx, &info);
+   info.offset = get_buf_entry(ctx, VIRGL_COPY_TRANSFER3D_SRC_RES_OFFSET);
+   info.synchronized = (get_buf_entry(ctx, VIRGL_COPY_TRANSFER3D_SYNCHRONIZED) != 0);
+
+   src_handle = get_buf_entry(ctx, VIRGL_COPY_TRANSFER3D_SRC_RES_HANDLE);
+
+   return vrend_renderer_copy_transfer3d(ctx->grctx, &info, src_handle);
+}
+
 void vrend_renderer_context_create_internal(uint32_t handle, uint32_t nlen,
                                             const char *debug_name)
 {
@@ -1355,7 +1487,9 @@ int vrend_decode_block(uint32_t ctx_id, uint32_t *block, int ndw)
          vrend_report_buffer_error(gdctx->grctx, 0);
          break;
       }
-//      fprintf(stderr,"[%d] cmd is %d (obj %d) len %d\n", gdctx->ds->buf_offset, header & 0xff, (header >> 8 & 0xff), (len));
+
+      VREND_DEBUG(dbg_cmd, gdctx->grctx,"%-4d %-20s len:%d\n",
+                  gdctx->ds->buf_offset, vrend_get_comand_name(header & 0xff), len);
 
       switch (header & 0xff) {
       case VIRGL_CCMD_CREATE_OBJECT:
@@ -1478,6 +1612,24 @@ int vrend_decode_block(uint32_t ctx_id, uint32_t *block, int ndw)
       case VIRGL_CCMD_TEXTURE_BARRIER:
          ret = vrend_decode_texture_barrier(gdctx, len);
          break;
+      case VIRGL_CCMD_SET_DEBUG_FLAGS:
+         ret = vrend_decode_set_debug_mask(gdctx, len);
+         break;
+      case VIRGL_CCMD_GET_QUERY_RESULT_QBO:
+         ret = vrend_decode_get_query_result_qbo(gdctx, len);
+         break;
+      case VIRGL_CCMD_TRANSFER3D:
+         ret = vrend_decode_transfer3d(gdctx, len, ctx_id);
+         break;
+      case VIRGL_CCMD_COPY_TRANSFER3D:
+         ret = vrend_decode_copy_transfer3d(gdctx, len);
+         break;
+      case VIRGL_CCMD_END_TRANSFERS:
+         ret = 0;
+         break;
+      case VIRGL_CCMD_SET_TWEAKS:
+         ret = vrend_decode_set_tweaks(gdctx, len);
+         break;
       default:
          ret = EINVAL;
       }
