@@ -21,9 +21,20 @@
  * OTHER DEALINGS IN THE SOFTWARE.
  *
  **************************************************************************/
+
 #ifndef VTEST_UTIL_H
 #define VTEST_UTIL_H
 
 int vtest_wait_for_fd_read(int fd);
 
+int __failed_call(const char* func, const char *called, int ret);
+
+int __failure(const char* func, const char *reason, int ret);
+
+#define report_failure(reason, ret) \
+   __failure(__func__, reason, ret)
+
+#define report_failed_call(called, ret) \
+   __failed_call(__func__, called, ret)
+
 #endif /* VTEST_UTIL_H */
